﻿namespace TabsStudioSorter
{
    class Marker
    {
        public delegate void HighlightIndexChangedDelegate(System.Windows.Controls.TabItem tabItem, System.Windows.Controls.Panel panel);
        public event HighlightIndexChangedDelegate OnHighlightIndexChanged;

        public Marker()
        {
            markerNotFound = false;
            openTabs = new System.Collections.Generic.List<TabInfo>();
        }

        public uint TabHighlightIndex(System.Windows.Controls.TabItem tabItem)
        {
            System.ComponentModel.DependencyPropertyDescriptor d = GetHighlightIndexDescriptor(tabItem);
            if (d == null)
                return 0;
            return (uint)d.GetValue(tabItem);
        }

        public void OnTabCreated(TabsStudioExt.TabEventArgs e)
        {
            TabInfo info = new TabInfo(e.Tab.TabItem,
                (object o, System.EventArgs a) =>
                { if (OnHighlightIndexChanged != null) OnHighlightIndexChanged(e.Tab.TabItem, e.Tabs.TabPanel2); });
            openTabs.Add(info);
            System.ComponentModel.DependencyPropertyDescriptor d = GetHighlightIndexDescriptor(e.Tab.TabItem);
            if (d != null)
                d.AddValueChanged(info.item, info.topChanged);
        }

        public void OnTabDestroyed(TabsStudioExt.TabEventArgs e)
        {
            foreach (TabInfo info in openTabs)
            {
                if (info.item == e.Tab.TabItem)
                {
                    System.ComponentModel.DependencyPropertyDescriptor d = GetHighlightIndexDescriptor(e.Tab.TabItem);
                    if (d != null)
                        d.RemoveValueChanged(info.item, info.topChanged);
                    openTabs.Remove(info);
                    return;
                }
            }
        }

        private System.ComponentModel.DependencyPropertyDescriptor GetHighlightIndexDescriptor(System.Windows.Controls.TabItem tabItem)
        {
            if (highlightIndexDescriptorCached == null && !markerNotFound)
            {
                markerNotFound = true;
                foreach (System.Reflection.Assembly a in System.AppDomain.CurrentDomain.GetAssemblies())
                {
                    if (a.FullName.StartsWith("Marker,"))
                    {
                        System.Type t = a.GetType("TabsStudioMarker.Properties");
                        if (t != null)
                            highlightIndexDescriptorCached = System.ComponentModel.DependencyPropertyDescriptor.FromName(
                                "HighlightIndex", t, tabItem.GetType());
                        break;
                    }
                }
            }
            return highlightIndexDescriptorCached;
        }

        private System.ComponentModel.DependencyPropertyDescriptor highlightIndexDescriptorCached;
        private bool markerNotFound;
        private System.Collections.Generic.List<TabInfo> openTabs;
    }
}
